""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('workflow', '0048_workflow_node_name'),
    ]

    HIST_WF_TBL = "historical_workflow"
    HIST_STEP_TBL = "historical_step"
    CREATE_IDX = "CREATE INDEX IF NOT EXISTS idx_{1}_{0} ON {0} ({1});"

    CREATE_PARTITION_FUNCTION = """
        CREATE OR REPLACE FUNCTION create_partitions(
            table_name TEXT,  
            partition_interval INTERVAL DEFAULT INTERVAL '1 day',
            num_partitions INT DEFAULT 1 -- Number of partitions to create, can be negative 
        )
        RETURNS VOID LANGUAGE plpgsql AS $$
        DECLARE
            partition_name TEXT;
            partition_start DATE := CURRENT_DATE;
            partition_end DATE;
        BEGIN
            -- Adjust the partition_start if num_partitions is negative (for past partitions)
            IF num_partitions < 0 THEN
                partition_start := partition_start + (num_partitions * partition_interval);
            END IF;
        
            FOR i IN 1..ABS(num_partitions) LOOP
                -- Calculate partition end date based on the provided interval
                partition_end := partition_start + partition_interval;
        
                -- Generate partition name: start date to end date minus one day for clarity
                partition_name := format('%I_%s_to_%s', table_name,  TO_CHAR(partition_start, 'YYYY_MM_DD'), TO_CHAR(partition_end - INTERVAL '1 day', 'YYYY_MM_DD'));
        
                -- Create the partition if it does not already exist
                EXECUTE format('CREATE TABLE IF NOT EXISTS %I PARTITION OF %I FOR VALUES FROM (''%s'') TO (''%s'')',
                    partition_name, table_name, partition_start, partition_end);
        
                partition_start := partition_end;
            END LOOP;
        END $$;"""

    CREATE_TABLE_HIST_WF = f"""
            CREATE TABLE IF NOT EXISTS public.{HIST_WF_TBL} (
            id integer NOT NULL,
            status character varying(30) NOT NULL,
            name character varying(500) NOT NULL,
            created timestamp with time zone NOT NULL,
            modified timestamp with time zone NOT NULL,
            template_iri character varying(800),
            args jsonb NOT NULL,
            result jsonb NOT NULL,
            env jsonb NOT NULL,
            parent_id integer,
            task_id character varying(500),
            debug boolean NOT NULL,
            parent_wf_id integer,
            records character varying(100)[],
            "user" character varying(500) NOT NULL,
            peer_details jsonb NOT NULL,
            owners character varying(100)[],
            tags character varying(1000),
            metadata_id integer,
            priority integer,
            node_name character varying(100),
            PRIMARY KEY (id, modified)
        ) PARTITION BY RANGE (modified);
    """

    CREATE_TABLE_HIST_STEP = f"""
        CREATE TABLE IF NOT EXISTS public.{HIST_STEP_TBL} (
            id integer NOT NULL,
            status character varying(30) NOT NULL,
            initial boolean NOT NULL,
            workflow_id integer,
            created timestamp with time zone NOT NULL,
            modified timestamp with time zone NOT NULL,
            name character varying(500) NOT NULL,
            template_iri character varying(800),
            args jsonb NOT NULL,
            func character varying(500) NOT NULL,
            result jsonb NOT NULL,
            task_id character varying(500),
            data jsonb NOT NULL,
            input jsonb NOT NULL,
            started timestamp with time zone,
            metadata_id integer,
            parent_id integer,
            completed timestamp with time zone,
            PRIMARY KEY (id, modified)
        ) PARTITION BY RANGE (modified);
    """

    operations = [
        migrations.RunSQL(CREATE_PARTITION_FUNCTION),
        migrations.RunSQL(CREATE_TABLE_HIST_WF),
        migrations.RunSQL(CREATE_TABLE_HIST_STEP),
        migrations.RunSQL(CREATE_IDX.format(HIST_WF_TBL, "status")),
        migrations.RunSQL(CREATE_IDX.format(HIST_WF_TBL, "name")),
        migrations.RunSQL(CREATE_IDX.format(HIST_WF_TBL, "parent_wf_id")),
        migrations.RunSQL(CREATE_IDX.format(HIST_STEP_TBL, "workflow_id")),
        migrations.RunSQL(CREATE_IDX.format(HIST_STEP_TBL, "parent_id")),
    ]
